/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 */

#include "config.h"
#include "ImageDecoder.h"
#include "NotImplemented.h"
#include "GraphicsTypes.h"
#include "GraphicsContextPlatformPrivate.h"
#include "StatisticsWiiU.h"

using namespace WTF;
namespace WebCore {
extern "C" {
	extern int gForce16bitAlphaTextures;
	extern int gForce16bitOpaqueTextures;
};

NativeImagePtr ImageFrame::asNewNativeImage(bool noCopy) const
{
	bool dither = false;
	NativeImagePtr texture;
	GraphicsContextPlatformPrivate::TextureFormat fmt = GraphicsContextPlatformPrivate::TEX_FMT_RGBA8;
	if(m_hasAlpha)
	{
		if(gForce16bitAlphaTextures)
		{
			fmt = GraphicsContextPlatformPrivate::TEX_FMT_RGBA4444;
			if(gForce16bitAlphaTextures > 1)
				dither = true;
		}
	}
	else
	{
		if(gForce16bitOpaqueTextures)
		{
			fmt = GraphicsContextPlatformPrivate::TEX_FMT_RGBA5551;
			if(gForce16bitOpaqueTextures > 1)
				dither = true;
		}
	}
	if(noCopy==false)
		texture = GraphicsContextPlatformPrivate::createTextureFromMemory(width(), height(), m_bytes, fmt, dither);
	else
		texture = GraphicsContextPlatformPrivate::createTextureFromMemoryOnly(width(), height(), m_bytes, fmt);
	STATS_WIIU_INC_IMAGE_COUNT();
    return texture;
}

void ImageFrame::copyToNativeImage(NativeImagePtr texture) const
{
	GraphicsContextPlatformPrivate::copyTextureFromMemoryToNative(texture, width(), height(), m_bytes);
}

} // namespace WebCore
