/*
 *  Copyright (C) 2008 INdT - Instituto Nokia de Tecnologia
 *  Copyright (C) 2009-2010 ProFUSION embedded systems
 *  Copyright (C) 2009-2010 Samsung Electronics
 *  Copyright (c) 2012 NINTENDO.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef ClipboardWiiU_h
#define ClipboardWiiU_h

#include "WiiUDataObject.h"
#include "CachedResourceClient.h"
#include "Clipboard.h"


namespace WebCore {
class CachedImage;
class Frame;
class IntPoint;

class ClipboardWiiU : public Clipboard, public CachedResourceClient {
    WTF_MAKE_FAST_ALLOCATED;
public:
    static PassRefPtr<ClipboardWiiU> create(
        ClipboardType, PassRefPtr<WiiUDataObject>, ClipboardAccessPolicy, Frame*);

    virtual  ~ClipboardWiiU();

    PassRefPtr<WiiUDataObject> dataObject()
    {
        return m_dataObject;
    }

    void clearData(const String&);
    void clearAllData();
    String getData(const String&, bool&) const;
    bool setData(const String&, const String&);

    HashSet<String> types() const;
    virtual PassRefPtr<FileList> files() const;

    void setDragImage(CachedImage*, const IntPoint&);
    void setDragImageElement(Node*, const IntPoint&);

    virtual DragImageRef createDragImage(IntPoint&) const;
    virtual void declareAndWriteDragImage(Element*, const KURL&, const String&, Frame*);
    virtual void writeURL(const KURL&, const String&, Frame*);
    virtual void writeRange(Range*, Frame*);

    virtual bool hasData();

    virtual void writePlainText(const WTF::String&);

#if ENABLE(DATA_TRANSFER_ITEMS)
    virtual PassRefPtr<DataTransferItems> items();
#endif

private:
    ClipboardWiiU(ClipboardType, PassRefPtr<WiiUDataObject>, ClipboardAccessPolicy, Frame*);
    void setDragImage(CachedImage*, Node*, const IntPoint&);
    RefPtr<WiiUDataObject> m_dataObject;
    Frame* m_frame;

    uint64_t m_originalSequenceNumber;
};
}

#endif
