/*---------------------------------------------------------------------------*

  Copyright 2010-2011 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
// -----------------------------------------------------------------------------
//  wkaudAudio.h
//
// -----------------------------------------------------------------------------

#ifndef __WKAUD_AUDIO_H__
#define __WKAUD_AUDIO_H__

//#if PLATFORM(WIIU)
//#include <types.h>
//#else
#include <stdint.h>
#include <stddef.h>

typedef int8_t   s8;
typedef int16_t  s16;
typedef int32_t  s32;
typedef int64_t  s64;
typedef uint8_t  u8;
typedef uint16_t u16;
typedef uint32_t u32;
typedef uint64_t u64;
//#endif

#ifdef __cplusplus
extern "C"
{
#endif

#define WKSUP_AUDIO_HANDLER_PRIO_MAX	0
#define WKSUP_AUDIO_HANDLER_PRIO_MIN	31

#define WKSUP_AUDIO_HANDLER_ADD			0
#define WKSUP_AUDIO_HANDLER_REMOVE		1

/// \brief Type of the application-provided audio callback registry to which audio clients requiring periodic callbacks can register themselves.
typedef void (*wksupAudioHandlerRegistry)(void (*pAudioCallback)(void *), void *pUserValue, s32 priority, s32 addOrRemove);

void wksupAudioStartup();
void wksupAudioShutdown();

/// \brief Registers a global audio callback registration function to which audio callback receivers can register themselves.
///
void wksupAudioSetRegistry(wksupAudioHandlerRegistry registry);

/// \brief Returns the a global audio callback registration function to which audio callback receivers can register themselves.
///
wksupAudioHandlerRegistry wksupAudioGetRegistry();

void wksupAudioSetOutputSamplingRate(u32 outputSampleRate);
u32 wksupAudioGetOutputSamplingRate();

void wksupAudioSetWebAudioRenderParams(u32 workerThreadStackSize, s32 workerThreadPriority, s32 workerThreadCoreAffinityMask);
void wksupAudioGetWebAudioRenderParams(u32 *workerThreadStackSize, s32 *workerThreadPriority, s32 *workerThreadCoreAffinityMask);

void *wksupAudioLCBeginUse(int bytes);
void wksupAudioLCEndUse(void);

void wksupAudioBeginSingleThread(void);
void wksupAudioEndSingleThread(void);

void wksupAudioLCSetBackground(int bIsBackground);

#ifdef __cplusplus
}
#endif

#endif /// __WKAUD_AUDIO_H__

