/*---------------------------------------------------------------------------*

  Copyright 2010-2011 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
// -----------------------------------------------------------------------------
//  wkaudAudioPlatform.h
//
// -----------------------------------------------------------------------------

#ifndef __WKAUDAUDIOPLATFORM_H__
#define __WKAUDAUDIOPLATFORM_H__

#if defined(_WIIU_)
#include <types.h>
#elif defined(WIN32) || defined(PF_UNIXBLD)
#if defined(WIN32)
#include <malloc.h>
#else
#include <stdlib.h>
#endif
typedef float f32;
#endif

namespace WkSupAudioUtilsPrivate {

typedef u32 uptr;

//
// Alignment
//
template <typename T> static T Align2(T val)  { return(T((uptr(val) + uptr(  1)) & ~uptr(  1))); }
template <typename T> static T Align4(T val)  { return(T((uptr(val) + uptr(  3)) & ~uptr(  3))); }
template <typename T> static T Align8(T val)  { return(T((uptr(val) + uptr(  7)) & ~uptr(  7))); }
template <typename T> static T Align16(T val) { return(T((uptr(val) + uptr( 15)) & ~uptr( 15))); }
template <typename T> static T Align32(T val) { return(T((uptr(val) + uptr( 31)) & ~uptr( 31))); }
template <typename T> static T Align64(T val) { return(T((uptr(val) + uptr( 63)) & ~uptr( 63))); }

//
// Pointer advancement
//
template <typename T, typename C> static T AdvancePtr(T pPointer, C numBytes) { return(T(uptr(pPointer) + uptr(numBytes))); }

//
// Endian swap
//
static inline u16 EndianSwap(u16 value)	{ return((value >> 8) | (value << 8)); }
static inline s16 EndianSwap(s16 value) { return(s16(EndianSwap(u16(value)))); }
static inline u32 EndianSwap(u32 value) { return((value << 24) | ((value & 0xff00) << 8) | ((value >> 8) & 0xff00) | (value >> 24)); }
static inline s32 EndianSwap(s32 value) { return(s32(EndianSwap(u32(value)))); }

// If we were running on a little endian platform this function should just be returning the input value
template <typename T> static inline T ConvIntelToNative(T value) { return EndianSwap(value); }

#define	endOfBufferGuardSamples	4

//
// Floating point sample data
//
struct PerChannelFloatSample {
	PerChannelFloatSample();
	~PerChannelFloatSample();

	enum AudioFormatError {
		eErr_Ok,							//!< No error
		eErr_UnexpectedFormat,				//!< Data not in the expected format
		eErr_BadFormat,						//!< Data corrupt
		eErr_NumChannels,					//!< Unsupported number of channels
		eErr_NumSampleBits,					//!< Unsupported number of bits per sample
		eErr_BadSampleRate,					//!< Unsupported sample rate
		eErr_Compression,					//!< Unsupported compression format
		eErr_OutOfMemory,					//!< Out of memory
		eErr_NotSupported,					//!< Format recognized, but not supported.
	};

	enum PerChannelDataFormat {
		format_none,
		format_PCM,
		format_DSPADPCM
	};

	AudioFormatError ConvertFromInMemoryData(const void *pData, u32 nNumBytes);

	enum { kMaxChannels = 8 };
	f32		*mpaChannelData[kMaxChannels];
	s16		*s16ChannelData[kMaxChannels];
	u32		mNumChannels;
	u32		mNumSamplesPerChannel;
	u32		mSampleRate;
	u32		mLoopStart;
	u32		mLoopEnd;
	u32		mNumSamplesAllocated[kMaxChannels];
	PerChannelDataFormat	mDataFormat;
};

//
// Sample provider for WebAudio
//
class WebAudioSampleProvider {
public:
	virtual ~WebAudioSampleProvider() { }
	virtual size_t ProvideRenderSamples(s16 **ppSampleReceiveBuffers, size_t nChannels, size_t nSamplesRequested) = 0;
};

//
// Audio renderer for WebAudio
//
class WebAudioRenderer {
public:
	WebAudioRenderer();
	virtual ~WebAudioRenderer();

	enum OutputDevice {
		eOutDev_TV,
		eOutDev_GAMEPAD,
	};

	bool OpenRenderer(OutputDevice onDevice, size_t nRenderFrequency, size_t nChannels, size_t nSampleRate, size_t nSamplesPerBlock, WebAudioSampleProvider *pSampleProvider);
	void CloseRenderer();

	void StartRenderer();
	void StopRenderer();

private:
	class Impl;
	Impl *mpImpl;
};

};	// WkSupAudioUtilsPrivate


#endif /// __WKAUDAUDIOPLATFORM_H__

