/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _WKGFX_CLIPOP_H_
#define _WKGFX_CLIPOP_H_

#include "PlatformPathWiiU.h"

namespace WKgfx {

class ClipOp {
	WKgfx::Matrix	m_viewMtx;
    PlatformPath *	m_pPath;
    float			m_x, m_y, m_width, m_height;
    bool 			m_bClipOut;

public:
    ClipOp() : m_pPath(NULL), m_bClipOut(false) { };

    ClipOp(const ClipOp &other)
    {
        if (other.m_pPath)
            m_pPath = new PlatformPath(*other.m_pPath); 
        else
		{
            m_pPath = NULL;
            m_x = other.m_x;
            m_y = other.m_y;
            m_width = other.m_width;    
            m_height = other.m_height;
        }

        m_bClipOut = other.m_bClipOut;
		m_viewMtx = other.m_viewMtx;
    }

    ClipOp(const PlatformPath & other, const WKgfx::Matrix & viewMtx, bool bClipOut = false) : m_viewMtx(viewMtx), m_bClipOut(bClipOut)
    { 
        m_pPath = new PlatformPath(other);
    }

    ClipOp(float x, float y, float width, float height, const WKgfx::Matrix & viewMtx, bool bClipOut = false) :
        m_viewMtx(viewMtx), m_pPath(NULL), m_x(x), m_y(y), m_width(width), m_height(height), m_bClipOut(bClipOut)
    { 
    }

    ~ClipOp()
    {
        if (m_pPath)
			delete m_pPath;
    }

    ClipOp & operator=(const ClipOp &other)
    {
        if (this != &other)
		{
			this->~ClipOp();
			new(this) ClipOp(other);
        }
	return(*this);
    }

    void apply(WebCore::GraphicsContext *ctx);

	bool isClipOut(void) const							{ return m_bClipOut; }
	const WKgfx::Matrix & getViewMatrix() const			{ return m_viewMtx; }
	void getExtents(WebCore::FloatRect & rect) const;
};

} // namespace WKgfx

#endif // _WKGFX_CLIPOP_H_
